@@ -63,6 +63,7 @@
 	session-data
 	define-record-types
 	threads threads-internal
+	queues
 	scheduler
 	interrupts
 	weak
@@ -408,12 +409,13 @@
 
 (define-structure placeholders placeholder-interface
   (open scheme-level-1 define-record-types
-	threads threads-internal
+	threads threads-internal queues
 	interrupts
 	signals)
   (files (big placeholder))
   (optimize auto-integrate))
 
+
 ;----------------
 ; Big Scheme
 
@@ -422,10 +424,6 @@
 	signals)		;call-error
   (files (big random)))
 
-(define-structure sort (export sort-list sort-list!)
-  (open scheme-level-2)
-  (files (big sort)))
-
 (define-structure pp (export p pretty-print define-indentation)
   (open scheme-level-2
         tables
@@ -494,6 +492,87 @@
 	threads thread-cells fluids)
   (files (big thread-fluid)))
 
+;;; Package defs for the Scheme Underground sorting package,
+
+;;; The general sort package:
+
+(define-structure sorting sorting-interface
+  (open scheme
+	list-merge-sort
+	vector-heap-sort
+	vector-merge-sort
+	sorted
+	delete-neighbor-duplicates)
+  (files (sort sort))
+  (optimize auto-integrate))
+
+(define-structure sorted sorted-interface
+  (open scheme
+	vector-utils)
+  (files (sort sortp))
+  (optimize auto-integrate))
+
+(define-structure delete-neighbor-duplicates delete-neighbor-duplicates-interface
+  (open scheme
+	receiving
+	vector-utils)
+  (files (sort delndups))
+  (optimize auto-integrate))
+
+(define-structure binary-searches binary-searches-interface
+  (open scheme
+	vector-utils)
+  (files (sort vbinsearch)))
+
+(define-structure list-merge-sort list-merge-sort-interface
+  (open scheme
+	receiving
+	(subset signals (error)))
+  (files (sort lmsort))
+  (optimize auto-integrate))
+
+(define-structure vector-merge-sort vector-merge-sort-interface
+  (open scheme
+	receiving
+	vector-utils
+	vector-insertion-sort-internal)
+  (files (sort vmsort))
+  (optimize auto-integrate))
+
+(define-structure vector-heap-sort vector-heap-sort-interface
+  (open scheme
+	receiving
+	vector-utils)
+  (files (sort vhsort))
+  (optimize auto-integrate))
+
+(define-structures ((vector-insertion-sort vector-insertion-sort-interface)
+		    (vector-insertion-sort-internal
+		     vector-insertion-sort-internal-interface))
+  (open scheme
+	vector-utils)
+  (files (sort visort))
+  (optimize auto-integrate))
+
+(define-structure vector-utils (export vector-copy
+				       vector-portion-copy
+				       vector-portion-copy!
+				       vector-start+end
+				       vectors-start+end-2)
+  (open scheme)
+  (files (sort vector-util)))
+
+;;; end Package defs for the Scheme Underground sorting package,
+
+(define-structure sort (export sort-list sort-list!)
+  (open scheme-level-2
+	sorting)
+  (begin
+    (define (sort-list l obj-<)
+      (list-sort obj-< l))
+    (define (sort-list! l obj-<)
+      (list-sort! obj-< l))))
+
 (define-structure big-util big-util-interface
   (open scheme-level-2
 	formats
@@ -662,6 +741,223 @@
 	features)		; make-immutable
   (files (big finite-type)))
 
+; nondeterminism via call/cc
+
+(define-structure nondeterminism (export with-nondeterminism
+					 ((either one-value all-values) :syntax)
+					 fail)
+  (open scheme-level-2
+	fluids cells
+	(subset signals (error)))
+  (files (big either)))
+
+;----------------
+; SRFI packages
+
+; SRFI-0 - Doesn't work with the module system.
+
+; Olin's list library.
+
+(define-structure srfi-1 srfi-1-interface
+  (open scheme-level-2
+	receiving
+	(subset signals (error)))
+  (files (srfi srfi-1)))
+
+(define-structure srfi-2 (export (and-let* :syntax))
+  (open scheme-level-2
+	signals)		; error
+  (files (srfi srfi-2)))
+
+; SRFI-3 - withdrawn
+; SRFI-4 - needs hacks to the reader
+
+(define-structure srfi-5 (export (let :syntax))
+  (open (modify scheme-level-2 (hide let)))
+  (files (srfi srfi-5)))
+
+(define-structure srfi-6 (export open-input-string
+				 open-output-string
+				 get-output-string)
+  (open (modify extended-ports
+		(rename (make-string-input-port    open-input-string)
+			(make-string-output-port   open-output-string)
+			(string-output-port-output get-output-string)))))
+
+; Configuration language
+
+(define-structure srfi-7 (export)	; defines a command
+  (open scheme
+
+	; for parsing programs
+	receiving
+	nondeterminism
+	(subset signals (error))
+
+	(subset package-commands-internal	(config-package))
+	ensures-loaded
+	(subset packages			(note-structure-name!))
+
+	; for defining the command
+	(subset command-processor		(define-user-command-syntax
+						 user-command-environment))
+	(subset environments			(environment-define!)))
+
+  (begin
+    (define available-srfis
+      '(srfi-1 srfi-2 srfi-5 srfi-6 srfi-7 srfi-8 srfi-9
+	srfi-11 srfi-13 srfi-14 srfi-16 srfi-17 srfi-19 srfi-23
+	srfi-25 srfi-26 srfi-27 srfi-28 
+        srfi-30 srfi-31	srfi-37
+        srfi-42))
+
+    ; Some SRFI's redefine Scheme variables.
+    (define shadowed
+      '((srfi-1 map for-each member assoc)
+	(srfi-5 let)
+	(srfi-13 string->list string-copy string-fill!)
+	(srfi-17 set!)))
+    )
+
+  (files (srfi srfi-7)))
+
+; Taken directly from the SRFI document (or from `receiving', take your pick).
+
+(define-structure srfi-8 (export (receive :syntax))
+  (open scheme-level-2)
+  (begin
+    (define-syntax receive
+      (syntax-rules ()
+	((receive formals expression body ...)
+	 (call-with-values (lambda () expression)
+			   (lambda formals body ...)))))))
+
+; SRFI-9 is a slight modification of DEFINE-RECORD-TYPE.
+
+(define-structure srfi-9 (export (define-record-type :syntax))
+  (open scheme-level-2 
+	(with-prefix define-record-types sys:))
+  (begin
+    (define-syntax define-record-type
+      (syntax-rules ()
+	((define-record-type type-name . stuff)
+	 (sys:define-record-type type-name type-name . stuff))))))
+
+; SRFI-10 - no stand-alone interface.
+
+(define-structure srfi-11 (export (let-values :syntax)
+				  (let*-values :syntax))
+  (open scheme-level-2)
+  (files (srfi srfi-11)))
+
+; SRFI-12 - withdrawn
+
+; Two more encyclopedias from Olin.
+
+(define-structure srfi-13 srfi-13-interface
+  (open scheme-level-2
+	bitwise
+	srfi-8 srfi-14
+	(subset signals		(error)))
+  (files (srfi srfi-13)))
+
+(define-structure srfi-14 srfi-14-interface
+  (open scheme-level-2
+	bitwise
+	srfi-9
+	(modify ascii (rename (char->ascii %char->latin1)
+			      (ascii->char %latin1->char)))
+	(subset features (make-immutable!))
+	(subset signals (error)))
+  (files (srfi srfi-14)))
+
+; SRFI-15 - withdrawn
+
+(define-structure srfi-16 (export (case-lambda :syntax))
+  (open scheme-level-2
+	(subset signals (error)))
+  (files (srfi srfi-16)))
+
+(define-structure srfi-17 (export (set! :syntax) setter)
+  (open (modify scheme-level-2 (rename (set! scheme-set!)))
+	(subset signals (error))
+	(subset util (unspecific)))
+  (files (srfi srfi-17)))
+
+; SRFI-18 - no implementation given
+
+(define (make-srfi-19 scheme-with-scsh)
+  (structure srfi-19-interface
+     (open scheme-with-scsh
+           (subset srfi-1 (reverse!))
+           srfi-6
+           srfi-8
+           signals
+           srfi-9)
+     (files (srfi srfi-19))))
+
+; SRFI-20 - withdrawn
+; SRFI-21 - no implementation given
+; SRFI-22 - no implementation given
+
+(define-structure srfi-23 (export error)
+  (open (subset signals (error))))
+
+; SRFI-24 - withdrawn
+
+(define-structure srfi-25 (export 
+			   array? make-array shape array 
+			   array-rank 
+			   array-start array-end 
+			   array-ref array-set! share-array)
+  (open scheme
+	srfi-23
+	srfi-9)
+  (files (srfi srfi-25)))
+  
+
+(define-structure srfi-26 (export ((cut cute) :syntax))
+  (open scheme)
+  (files (srfi srfi-26)))
+
+(define-structure srfi-27 srfi-27-interface
+  (open
+   scheme-level-1
+   floatnums
+   external-calls
+   (subset srfi-9 (define-record-type)) 
+   (subset srfi-23 (error)))
+;; scsh doesn't have S48's posix subsystem yet:
+;   (subset posix-time (current-time)) 
+;   (subset posix (time-seconds)))
+  (files (srfi srfi-27)))
+
+(define-structure srfi-28 (export format)
+  (open scheme
+	srfi-23
+	srfi-6)
+  (files (srfi srfi-28)))
+
+; SRFI-29 - requires access to the current locale
+
+; SRFI-30 - scheme/rts/read.scm contains the reader for #|...|# comments
+
+(define-structure srfi-31 srfi-31-interface
+  (open scheme)
+  (files (srfi srfi-31)))
+
+(define-structure srfi-37 srfi-37-interface
+  (open scheme 
+	srfi-9 
+	srfi-11)
+  (files (srfi srfi-37)))
+
+; Eager Comprehensions
+
+(define-structure srfi-42 srfi-42-interface
+  (open scheme
+	srfi-23)
+  (files (srfi srfi-42)))
 ; ... end of package definitions.
 
 ; Temporary compatibility stuff
@@ -687,6 +983,7 @@
 	    bignums ratnums recnums floatnums
 	    build
 	    callback
+	    cells
 	    command-levels
 	    command-processor
 	    debugging
@@ -723,7 +1020,17 @@
 	    search-trees
 	    sicp
 	    sockets
+
 	    sort
+	    delete-neighbor-duplicates
+	    binary-searches
+	    sorted
+	    list-merge-sort
+	    vector-merge-sort
+	    vector-heap-sort
+	    vector-insertion-sort
+	    sorting
+
 	    strong
 	    thread-fluids
 	    traverse
@@ -742,6 +1049,14 @@
 	    ;; Compatibility
 	    record table
 	    build-internals                  ;added by JMG
+
+            ; SRFI packages
+	    srfi-1 srfi-2 srfi-5 srfi-6 srfi-7 srfi-8 srfi-9
+	    srfi-11 srfi-13 srfi-14 srfi-16 srfi-17
+	    srfi-23 srfi-25 srfi-26 srfi-27 srfi-28
+	    srfi-31 srfi-37
+            srfi-42
 	    )
 	   :structure)
+          make-srfi-19
 	  ((define-signature define-package) :syntax)))
