@@ -51,7 +51,7 @@ s48_value scheme_bind(s48_value sock, s48_value family, s48_value scheme_name)
 		scheme_length);	/* copy to c string */
 	name.sun_path[scheme_length]='\0'; /* add null */
 	if ( bind(sockfd,(struct sockaddr *)&name,sizeof(name)) < 0)
-	  s48_raise_os_error (errno);
+	  s48_raise_os_error_3 (errno, sock, family, scheme_name);
 	return S48_UNSPECIFIC;
       }
     case AF_INET: 
@@ -65,7 +65,7 @@ s48_value scheme_bind(s48_value sock, s48_value family, s48_value scheme_name)
 	name.sin_addr.s_addr=addr;
 	name.sin_port=port;
 	if (bind(sockfd,(struct sockaddr *)&name,sizeof(name)) < 0)
-	  s48_raise_os_error (errno);
+	  s48_raise_os_error_3(errno, sock, family, scheme_name);
 
 	return S48_UNSPECIFIC;
       }
@@ -83,33 +83,25 @@ s48_value scheme_connect(s48_value sock, s48_value family, s48_value scheme_name
     case AF_UNIX: 
       {
 	struct sockaddr_un name;
-	int ret;
 	int scheme_length=S48_STRING_LENGTH(scheme_name);
 	
 	memset(&name, 0, sizeof(name));
 	
 	name.sun_family=AF_UNIX;	
-	if (scheme_length>=(108-1)) /* save space for \0 */
-	  return(-1);
+
 	strncpy(name.sun_path,
 		s48_extract_string (scheme_name),
 		scheme_length);	/* copy to c string */
 	name.sun_path[scheme_length]='\0'; /* add null */
 	
 	if (connect(sockfd,(struct sockaddr *)&name,sizeof(name)) == 0)
-	  return S48_TRUE;
+	  return s48_cons (S48_FALSE, S48_UNSPECIFIC);
 	
 	if (errno != EWOULDBLOCK && errno != EINTR && errno != EALREADY
 	    && errno != EINPROGRESS && errno != EAGAIN)
-	  s48_raise_os_error(errno);
+	  s48_raise_os_error_3(errno, sock, family, scheme_name);
 	 
-	if (! (s48_add_pending_fd(sockfd, 0)))
-	  s48_raise_out_of_memory_error();
-	
-	if (errno == EINPROGRESS) 
-	  return s48_enter_fixnum (0); 
-	else return S48_FALSE;
-	
+	return s48_cons (S48_TRUE, (errno == EINPROGRESS) ? S48_TRUE : S48_FALSE);	
       }
     case AF_INET: 
       {
@@ -125,19 +117,13 @@ s48_value scheme_connect(s48_value sock, s48_value family, s48_value scheme_name
 	name.sin_port=port;
 	
 	if (connect(sockfd,(struct sockaddr *)&name,sizeof(name)) == 0)
-	  return S48_TRUE;
+	  return s48_cons (S48_FALSE, S48_UNSPECIFIC);
 	
 	if (errno != EWOULDBLOCK && errno != EINTR && errno != EALREADY
 	    && errno != EINPROGRESS && errno != EAGAIN)
-	  s48_raise_os_error(errno);
-
-	if (! (s48_add_pending_fd(sockfd, 0)))
-	  s48_raise_out_of_memory_error();
-	
-	if (errno == EINPROGRESS) 
-	  return s48_enter_fixnum (0); 
-	else return S48_FALSE;
+	  s48_raise_os_error_3(errno, sock, family, scheme_name);
 	
+	return s48_cons (S48_TRUE, (errno == EINPROGRESS) ? S48_TRUE : S48_FALSE);
       }
     default: 
       s48_raise_argument_type_error (family);	/* error unknown address family */ 
@@ -168,13 +154,14 @@ s48_value scheme_accept(s48_value sockfd_tagged, s48_value family)
 	if (newsockfd < 0)
 	  {
 	    if ((errno != EWOULDBLOCK) && (errno != EINTR) && (errno != EAGAIN))
-	      s48_raise_os_error(errno);
-	    if (! s48_add_pending_fd(sockfd, 1))// 1 for is_input
+	      s48_raise_os_error_2(errno, sockfd_tagged, family);
+	    if (! s48_add_pending_fd(sockfd, 1))/* 1 for is_input */
 	      s48_raise_out_of_memory_error();
 	    return S48_FALSE;
 	  }
-	
-	return(s48_enter_fixnum (newsockfd));
+	fcntl(newsockfd, F_SETFL, O_NONBLOCK);
+	return(s48_cons (s48_enter_fixnum (newsockfd),
+			 s48_enter_string (name.sun_path)));
 	break;
       }
     case AF_INET: 
@@ -190,8 +177,8 @@ s48_value scheme_accept(s48_value sockfd_tagged, s48_value family)
 	if (newsockfd < 0)
 	  {
 	    if ((errno != EWOULDBLOCK) && (errno != EINTR) && (errno != EAGAIN))
-	      s48_raise_os_error(errno);
-	    if (! s48_add_pending_fd(sockfd, 1))// 1 for is_input
+	      s48_raise_os_error_2(errno, sockfd_tagged, family);
+	    if (! s48_add_pending_fd(sockfd, 1))/* 1 for is_input */
 	      s48_raise_out_of_memory_error();
 	    return S48_FALSE;
 	  }
@@ -236,7 +223,7 @@ s48_value scheme_peer_name(s48_value sock, s48_value family)
 	socklen_t namelen=sizeof(name);
 	int value=getpeername(sockfd,(struct sockaddr *)&name,&namelen);
 	
-	if (value < 0) s48_raise_os_error (errno);
+	if (value < 0) s48_raise_os_error_2(errno, sock, family);
 	
 	return (make_addr (name.sin_addr.s_addr, name.sin_port));
 	break;
@@ -267,7 +254,7 @@ s48_value scheme_socket_name(s48_value sock, s48_value family)
 	socklen_t namelen=sizeof(name);
 	int value=getsockname(sockfd,(struct sockaddr *)&name,&namelen);
 	
-	if (value < 0) s48_raise_os_error (errno);
+	if (value < 0) s48_raise_os_error_2(errno, sock, family);
 	  
 	return(make_addr (name.sin_addr.s_addr,
 			  name.sin_port));
@@ -305,7 +292,6 @@ recv_substring(s48_value scm_sockfd, s48_value flags, s48_value buf,
 		  buf_part, end-start, 
 		  s48_extract_fixnum (flags),
 		  (struct sockaddr *)&name, &namelen);
-  s48_value result;
   
   if (cc >= 0)
     return (s48_cons (s48_enter_fixnum (cc), 
@@ -315,8 +301,8 @@ recv_substring(s48_value scm_sockfd, s48_value flags, s48_value buf,
 	    
   
   if ((errno != EWOULDBLOCK) && (errno != EINTR) && (errno != EAGAIN))
-    s48_raise_os_error(errno);
-  if (! s48_add_pending_fd(sockfd, 1))// 1 for is_input
+    s48_raise_os_error_5(errno, scm_sockfd, flags, buf, scm_start, scm_end);
+  if (! s48_add_pending_fd(sockfd, 1))/* 1 for is_input */
     s48_raise_out_of_memory_error();
 	return S48_FALSE;
 }
@@ -391,9 +377,16 @@ s48_value send_substring(s48_value scm_sockfd,
     return s48_enter_fixnum (n);
 
   if ((errno != EWOULDBLOCK) && (errno != EINTR) && (errno != EAGAIN))
-    s48_raise_os_error(errno);
+    s48_raise_os_error_7(errno,
+			 scm_sockfd,
+			 scm_flags,
+			 buf, 
+			 scm_start, 
+			 scm_end, 
+			 scm_family, 
+			 scheme_name);
   
-  if (! s48_add_pending_fd(s, 0))// 0 for is_input
+  if (! s48_add_pending_fd(s, 0))/* 0 for is_input */
     s48_raise_out_of_memory_error();
   
   return S48_FALSE;
@@ -407,7 +400,7 @@ s48_value scheme_getsockopt (s48_value s,
 			     s48_value optname)
 {
   int optval;
-  int optlen=sizeof(optval);
+  size_t optlen=sizeof(optval);
   
   if (getsockopt(s48_extract_fixnum (s),
 		 s48_extract_fixnum (level),
@@ -424,7 +417,7 @@ s48_value scheme_getsockopt_linger (s48_value s,
 				    s48_value optname)
 {
   struct linger optval;
-  int optlen=sizeof(optval);
+  size_t optlen=sizeof(optval);
   
   if (getsockopt(s48_extract_fixnum (s),
 		 s48_extract_fixnum (level),
@@ -640,7 +633,7 @@ s48_value netent2net_info(struct netent *net)
 s48_value scheme_net_address2net_info(s48_value net_addr)
 {
   struct netent *net;
-  // expects host byte order :
+  /* expects host byte order */
   net=getnetbyaddr(s48_extract_unsigned_integer (net_addr),AF_INET); 
   
   return netent2net_info (net);
