@@ -140,10 +140,8 @@
 			      seek/delta
 			      seek/end
 
-;			      select
-;			      select!
-
 			      flush-all-ports
+			      flush-all-ports-no-threads
 			      y-or-n?
 			      *y-or-n-eof-count*
 			      ;; R4RS I/O procedures that scsh provides.
@@ -191,9 +189,12 @@
 			      (with-current-output-port :syntax)
 			      with-current-error-port*
 			      (with-current-error-port :syntax)
+			      with-error-output-port*
+			      (with-error-output-port :syntax)
 			      set-current-input-port!
 			      set-current-output-port!
 			      set-current-error-port!
+			      set-error-output-port!
 
 			      stdports->stdio
 			      stdio->stdports
@@ -218,6 +219,10 @@
 			      read-string!
 			      read-string/partial
 			      read-string!/partial
+
+			      select select!
+			      select-ports select-port-channels
+
 			      (write-string (proc (:string &opt :value :exact-integer :exact-integer) :unspecific))
 			      write-string/partial)))
 
@@ -270,6 +275,13 @@
 	  file-regular?
 	  file-socket?
 	  file-special?
+	  file-info-directory?
+	  file-info-fifo?
+	  file-info-regular?
+	  file-info-socket?
+	  file-info-special?
+	  file-info-symlink?
+
 	  file-not-readable?
 	  file-not-writable?
 	  file-not-writeable?	; Deprecated
@@ -278,16 +290,27 @@
 	  file-writeable?
 	  file-writable?	; Deprecated
 	  file-executable?
+	  file-info-not-readable?
+	  file-info-not-writable?
+	  file-info-not-executable?
+	  file-info-readable?
+	  file-info-writable?
+	  file-info-executable?
+
 	  file-not-exists?
 	  file-exists?
 
 	  sync-file
 	  sync-file-system
 
+	  open-directory-stream
+	  close-directory-stream
+	  read-directory-stream
+	  
 	  directory-files
 	  glob
 	  glob-quote
-;	  file-match
+	  file-match
 
 	  create-temp-file
 	  temp-file-iterate
@@ -337,21 +360,21 @@
 
 
 (define-interface scsh-process-state-interface
-  (export umask
+  (export with-resources-aligned
+	  
+	  umask
 	  set-umask
 	  with-umask*
 	  (with-umask :syntax)
-	  with-umask-aligned*
-	  (with-umask-aligned :syntax)
-
+	  umask-resource
+	  
 	  process-chdir
 	  process-cwd
 	  chdir
 	  cwd
 	  with-cwd*
 	  (with-cwd :syntax)
-	  with-cwd-aligned*
-	  (with-cwd-aligned :syntax)
+	  cwd-resource
 
 	  pid
 	  parent-pid
@@ -367,7 +390,14 @@
 	  user-supplementary-gids
 	  set-uid
 	  set-gid
-	  set-effective-gid
+	  set-user-effective-uid
+	  set-user-effective-gid
+	  with-user-effective-uid*
+	  with-user-effective-gid*
+	  ((with-user-effective-uid 
+	    with-user-effective-gid):syntax)
+	  euid-resource
+	  egid-resource
 
 	  system-name
 	  process-times
@@ -425,7 +455,8 @@
 	  (with-env :syntax)
 	  (with-total-env :syntax)
 	  add-before
-	  add-after))
+	  add-after
+	  environ-resource))
 
 
 (define-interface scsh-home-interface
@@ -566,15 +597,20 @@
 	  run/sexps*
 	  run/strings*
 
-	  char-filter
-	  string-filter))
+	  make-char-port-filter
+	  make-string-port-filter))
 
 
 (define-interface scsh-version-interface
   (export scsh-major-version
 	  scsh-minor-version
-	  scsh-version-string))
+	  scsh-version-string
+	  scsh-release-name))
 
+(define-interface exit-hooks-interface
+  (export call-exit-hooks-and-narrow
+	  add-narrowed-exit-hook!
+	  add-exit-hook!))
 
 ;;; This is probably bogus.
 (define-interface string-ports-interface
@@ -583,19 +619,26 @@
 	  make-string-output-port
 	  string-output-port-output))
 
+(define-interface string-collectors-interface
+  (export make-string-collector
+	  collect-string!
+	  clear-string-collector!
+	  string-collector->string))
+
+(define-interface delimited-readers-interface
+  (export read-delimited read-delimited! %read-delimited!
+	  read-line read-paragraph
+	  skip-char-set))
 
 (define-interface scsh-utilities-interface
-  (export del delete filter first first? nth
-	  fold fold-right
-	  any every
-	  mapv mapv! vector-every? copy-vector initialize-vector vector-append
+  (export mapv mapv! vector-every? copy-vector initialize-vector vector-append
 	  vfold vfold-right
-	  check-arg conjoin disjoin negate compose reverse! call/cc
+	  check-arg
 	  deprecated-proc
-	  deposit-bit-field
 	  real->exact-integer
 	  make-reinitializer
-	  run-as-long-as))
+	  run-as-long-as
+	  obtain-all-or-none))
 
 (define-interface weak-tables-interface
   (export make-weak-table weak-table-set! weak-table-ref weak-table-walk
@@ -642,10 +685,17 @@
 	  options/linger
 	  options/timeout))
 
+(define-interface scsh-endian-interface
+  (export net-to-host-32
+	  net-to-host-16
+	  host-to-net-32
+	  host-to-net-16))
+
 ;;; actual functions interface
 (define-interface scsh-sockets-interface
   (export socket-connect
 	  bind-listen-accept-loop
+	  bind-prepare-listen-accept-loop
 	  socket?
 	  socket:family
 	  socket:inport
@@ -661,6 +711,8 @@
 	  close-socket
 	  bind-socket
 	  connect-socket
+	  connect-socket-no-wait
+	  connect-socket-successful?
 	  listen-socket
 	  accept-connection
 	  socket-remote-address
@@ -696,21 +748,13 @@
 	  protocol-info?
 	  protocol-info:name
 	  protocol-info:aliases
-	  protocol-info:number
-	  
-	  net-to-host-32
-	  net-to-host-16
-	  host-to-net-32
-	  host-to-net-16
-	  ))
-
+	  protocol-info:number))
 
 (define-interface scsh-field-reader-interface
   (export join-strings
 	  field-splitter infix-splitter suffix-splitter sloppy-suffix-splitter
 	  record-reader
-	  field-reader
-	  nth))		; Kinda handy.
+	  field-reader))
 
 (define-interface scsh-delimited-readers-interface
   (export read-line
@@ -1119,4 +1163,61 @@
 	  with-syslog-channel))
 
 (define-interface crypt-interface
-  (export crypt))
\ No newline at end of file
+  (export crypt))
+
+(define-interface uname-interface
+  (export uname
+	  uname:os-name
+	  uname:node-name
+	  uname:release
+	  uname:version
+	  uname:machine
+	  type/uname))
+
+(define-interface md5-interface
+  (export make-md5-context
+	  md5-context?
+	  update-md5-context!
+	  md5-context->md5-digest
+			      
+	  md5-digest?
+
+	  md5-digest->number
+	  number->md5-digest
+
+	  md5-digest-for-string
+	  md5-digest-for-port))
+
+(define-interface configure-interface
+  (export host
+          machine
+          vendor
+          os
+          prefix
+          exec-prefix
+          bin-dir
+          lib-dir
+          include-dir
+          man-dir
+          lib-dirs-list
+          libs
+          defs
+          cflags
+          cppflags
+          ldflags
+          linker-flags
+          compiler-flags))
+
+(define-interface lib-dirs-interface
+  (export lib-dirs
+          find-library-file
+
+          lib-dirs-append-script-dir!
+          lib-dirs-prepend-script-dir!
+          reset-lib-dirs!
+          clear-lib-dirs!
+          lib-dirs-prepend!
+          lib-dirs-append!))
+
+(define-interface lib-dirs-internal-interface
+  (export expand-lib-dir))
