@@ -35,7 +35,7 @@
 (define-structure scsh-utilities scsh-utilities-interface
   (open bitwise error-package loopholes let-opt scheme define-record-types 
 	records
-	threads threads-internal placeholders)
+	threads threads-internal placeholders locks srfi-1)
   (files utilities)
 ;  (optimize auto-integrate)
   )
@@ -46,6 +46,28 @@
 	tables)
   (files weaktables))
 
+(define-structure string-collectors string-collectors-interface
+  (open scheme
+	defrec-package)
+  (files stringcoll))
+
+(define-structure delimited-readers delimited-readers-interface
+  (open scheme
+	byte-vectors
+	signals ; ERROR
+	let-opt
+	receiving
+	re-level-0 rx-syntax
+	(subset srfi-14 (char-set x->char-set char-set-contains?))
+	ascii
+	i/o-internal ports)
+  (files rdelim))
+
+(define list-lib srfi-1)
+(define string-lib srfi-13)
+(define char-set-lib srfi-14)
+
+
 ;;; This guy goes into the FOR-SYNTAX part of scsh's syntax exports.
 (define-structure scsh-syntax-helpers
   (export transcribe-extended-process-form)
@@ -81,16 +103,10 @@
   (open scheme)
   (files scsh-version))
 
-(define-structure partial-s48-ports
-  (export current-input-port current-output-port
-	  newline
-	  current-error-port
-
-	  $current-input-port
-	  $current-output-port
-	  $current-error-port
-	  $current-noise-port)
-  (open ports i/o))
+(define-structure scsh-endian scsh-endian-interface
+  (open scheme
+	bitwise)
+  (files endian))
 
 ;;; The scsh-level-0 package is for implementation convenience.
 ;;; The scsh startup and top-level modules need access to scsh
@@ -123,9 +139,11 @@
 			scsh-high-level-process-interface
 			scsh-time-interface ; new in 0.2
 			scsh-sockets-interface ; new in 0.3
+			scsh-endian-interface
 			tty-interface ; new in 0.4
 			scsh-version-interface
-			char-set-interface
+			(interface-of srfi-14) ;; export this here for
+			(export ->char-set)    ;; this kludge
 			signal-handler-interface
 			;; This stuff would probably be better off kept
 			;; in separate modules, but we'll toss it in for now.
@@ -133,6 +151,7 @@
 			string-ports-interface
 			syslog-interface
 			crypt-interface
+			uname-interface
 			))
    (scsh-level-0-internals (export set-command-line-args!
 				   init-scsh-hindbrain
@@ -150,27 +169,29 @@
 	defrec-package
 	define-record-types
 	formats
+	string-collectors
+	delimited-readers
 	os-dependent		; OS dependent stuff
 	buffered-io-flags	; stdio dependent
 	ascii
 	records
 	extended-ports
-	partial-s48-ports
 	ports
 	build
 	bigbit
 	bitwise
 	signals
 	conditions
+	(subset srfi-1 (filter reverse! fold delete any))
 	scsh-utilities
 	handle
 	fluids thread-fluids
 	weak-tables
+        (subset srfi-1 (last drop-right))
 
-	scsh-char-set-low-level-lib         ; rdelim.scm needs it.
+	srfi-14
 ;	scsh-regexp-package
 ;	scsh-regexp-internals
-	char-set-lib
 	scsh-version
 	tty-flags
 	scsh-internal-tty-flags	; Not exported
@@ -184,15 +205,21 @@
 	re-level-0
 	rx-syntax
 
-	string-lib
+	srfi-13
 
-	fluids			; For exec-path-list
+	thread-fluids			; For exec-path-list
 	loopholes		; For my bogus CALL-TERMINALLY implementation.
 
-	scheme
-	
+	(modify scheme (hide call-with-input-file
+			     call-with-output-file
+			     with-input-from-file
+			     with-output-to-file
+			     open-input-file
+			     open-output-file))
+			     
 	low-interrupt            ; for sighandler and procobj
-	i/o                  ; all these seem to be for scsh-0.6 JMG
+	;; all these seem to be for scsh-0.6 JMG
+	i/o
 	i/o-internal
 	channels channel-i/o
 	low-channels
@@ -203,7 +230,11 @@
 	command-levels
 	features
 	general-tables
-	simple-syntax)  
+	simple-syntax
+	exit-hooks
+        display-conditions
+
+	scsh-endian)  
   (for-syntax (open scsh-syntax-helpers scheme))
   (access interrupts
 	  sort
@@ -216,11 +247,9 @@
 	  records	; I don't think this is necessary. !!!
 	  scheme)	; For accessing the normal I/O operators.
   (files syntax
+         scsh-condition
 	 syscalls
-;	 select
 	 fname
-	 stringcoll
-	 scsh-condition
 	 rw
 	 newports
 	 fdports
@@ -229,20 +258,21 @@
 	 filesys
 	 fileinfo
 	 glob
-;	 filemtch
+	 filemtch
 	 time			; New in release 0.2.
 	 (machine time_dep)
 	 network		; New in release 0.3.
-	 endian			; New in release 0.4.
 	 flock			; New in release 0.4.
 	 tty			; New in release 0.4.
 	 pty			; New in release 0.4.
 	 sighandlers		; New in release 0.5.
 	 scsh
 ;	 re
-	 rdelim  
 	 )
 ;  (optimize auto-integrate)
+  (begin
+    ;; work around for SRFI 14 naming fuckage
+    (define ->char-set x->char-set))
   )
 
 (define-structure defrec-package (export (define-record :syntax))
@@ -283,18 +313,28 @@
 	usual-resumer           ; usual-resumer
 	environments            ; with-interaction-environment
 	fluids-internal            ; JMG: get-dynamic-env 
+	threads threads-internal queues scheduler
+	structure-refs
 	scsh-utilities
 	interrupts
 	low-interrupt
 	sigevents
 	primitives
-	scheme)
+	(modify scheme (hide call-with-input-file
+			     call-with-output-file
+			     with-input-from-file
+			     with-output-to-file
+			     open-input-file
+			     open-output-file)))
+  (access threads-internal)
   (files startup))
 
-(define-structure scsh-top-package (export parse-switches-and-execute 
-					   repl )
+(define-structure scsh-top-package (export parse-switches-and-execute
+					   with-scsh-initialized)
   (open command-processor
 	command-levels          ; with-new-session
+	conditions
+        display-conditions
 	ensures-loaded
 	environments
 	error-package
@@ -318,19 +358,52 @@
 				; with-current-output-port exit
 	scsh-level-0-internals	; set-command-line-args! init-scsh-vars
 	threads
-	list-lib                ; any
+        lib-dirs
+        lib-dirs-internal
+	(subset srfi-14 (char-set
+			 char-set-complement!
+			 char-set-contains?
+			 string->char-set))
 	root-scheduler          ; scheme-exit-now
+	exit-hooks
 	scheme)
   (files top meta-arg))
 
+(define-structure exit-hooks exit-hooks-interface
+  (open scheme
+	threads)
+  (begin
+    (define *exit-hooks* '())
+    (define (add-exit-hook! thunk)
+      (set! *exit-hooks* (cons thunk *exit-hooks*)))
+    (define (call-exit-hooks!)
+      (for-each (lambda (thunk) (thunk)) *exit-hooks*))
+    
+    (define *narrowed-exit-hooks* '())
+    (define (add-narrowed-exit-hook! thunk)
+      (set! *narrowed-exit-hooks* (cons thunk *narrowed-exit-hooks*)))
+    (define (call-narrowed-exit-hooks!)
+      (for-each (lambda (thunk) (thunk)) *narrowed-exit-hooks*))
+    
+    (define (call-exit-hooks-and-narrow thunk)
+      (call-exit-hooks!)
+      (narrow
+       (lambda ()
+	 (call-narrowed-exit-hooks!)
+	 (thunk))))))
+
 
 (define-structure field-reader-package scsh-field-reader-interface
   (open receiving             ; receive
-	scsh-utilities          ; nth
+	scsh-utilities          ; deprecated-proc
 	error-package		; error
-	string-lib		; string-join for obsolete join-strings
-	scsh-level-0		; delimited readers
-;	scsh-regexp-package
+	(subset srfi-13 (string-join))
+	(subset srfi-14 (char-set?
+			 char-set:whitespace
+			 char-set
+			 x->char-set
+			 char-set-complement))
+	delimited-readers
 	re-exports
 	let-opt			; optional-arg parsing & defaulting
 	scheme
@@ -347,7 +420,8 @@
    (awk-support-package (export next-range next-:range
 				next-range: next-:range:)))
   (open receiving		; receive
-	scsh-utilities	
+	;; scsh-utilities	
+	(subset srfi-1 (any filter))
 	error-package		; error
 ;	scsh-regexp-package
 ;	re-exports
@@ -382,15 +456,15 @@
 		      re-old-funs-interface
 		      scsh-field-reader-interface 	; new in 0.3
 ;		      scsh-dbm-interface
-		      (export repl)
 		      awk-interface
 		      char-predicates-interface; Urk -- Some of this is R5RS!
-		      obsolete-char-set-interface
 		      dot-locking-interface
+		      md5-interface
+                      configure-interface
+                      lib-dirs-interface
 		      )
 
   (open structure-refs
-	obsolete-char-set-lib
 	scsh-level-0
 	scsh-level-0-internals
 	re-exports
@@ -402,18 +476,32 @@
 	field-reader-package
 	char-predicates-lib	; Urk -- Some of this is R5RS!
 	dot-locking
+	md5
+        configure
+        lib-dirs
 	scheme)
 
   (access scsh-top-package)
-  (begin (define repl (structure-ref scsh-top-package repl)))
 ;  (optimize auto-integrate)
   )
 
+(define-structure scheme-with-scsh
+  (compound-interface (interface-of scsh)
+		      (interface-of scheme))
+  (open scsh
+	(modify scheme (hide call-with-input-file
+			     call-with-output-file
+			     with-input-from-file
+			     with-output-to-file
+			     open-input-file
+			     open-output-file))))
+
 (define-structure scsh-here-string-hax (export)
   (open reading
 	receiving
 	scsh		; Just need the delimited readers.
 	features	; make-immutable!
+	(subset srfi-14 (char-set))
 	scheme)
   (files here))
 
@@ -477,3 +565,44 @@
 	bitwise)
   (files syslog))
 
+(define-structure libscsh (export dump-libscsh-image)
+  (open scheme
+ 	external-calls 
+ 	(subset i/o (current-error-port))
+ 	(subset extended-ports (make-string-input-port))
+ 	(subset handle (with-handler))
+ 	(subset escapes (with-continuation))
+	(subset environments (with-interaction-environment))
+ 	(subset package-commands-internal (user-environment))
+ 	(subset command-levels (user-context start-new-session))
+ 	(subset command-processor (user-command-environment))
+	(subset scsh-startup-package (dump-scsh-program)))
+  (files libscsh))
+
+(define-structure md5 md5-interface
+  (open scheme
+	ascii
+	define-record-types
+	bitwise
+	(subset i/o (read-block))
+	(subset srfi-13 (string-fold-right))
+	signals
+	external-calls)
+  (files md5))
+
+(define srfi-19 (make-srfi-19 scheme-with-scsh))
+
+(define-structure configure configure-interface
+  (open scheme
+        re-level-0 rx-syntax
+        (subset srfi-13 (string-join)))
+  (files configure))
+
+(define-structures ((lib-dirs lib-dirs-interface)
+                    (lib-dirs-internal lib-dirs-internal-interface))
+  (open scsh-level-0
+        scheme
+        handle
+        scsh-utilities
+        (subset srfi-1 (any)))
+  (files lib-dirs))
\ No newline at end of file
